<?php

namespace FM\Service;

use FM\Factory\Fm00005000VistoriasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\Remote\cURLRequest;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;

class Fm00005000VistoriasService
{

    protected $connection;
    protected $Fm00005000VistoriasFactory;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, Fm00005000VistoriasFactory $Fm00005000VistoriasFactory, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->Fm00005000VistoriasFactory   = $Fm00005000VistoriasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    // Busca as Vistorias do Grid
    public function getVistorias()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        return $this->Fm00005000VistoriasFactory->getVistorias($listaOperadores);
    }

    // Buscar informaçãos para mondar o TreeTekGrid
    public function getCriteriosGrid($parametro)
    {
        $params = array(
            'NRGERENCIAMENTOCHK' => $parametro['NRGERENCIAMENTOCHK']
        );
        return $this->Fm00005000VistoriasFactory->getCriteriosGrid($params);
    }

    // Edita a vistoria
    public function updateInspection($param)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();
            if (!$parametros['finalizar']) {
                foreach ($parametros["DRG"] as $parametro) {
                    $salvaCriterio = array(
                        "IDMARCADO"             => $parametro['IDMARCADO'],
                        "NRCRITERIOTARGERCHK"   => $parametro['NR'],
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg,
                        "NRGERENCIAMENTOCHK"    => $parametro['NRGERENCIAMENTOCHK']

                    );

                    $this->Fm00005000VistoriasFactory->salvarCriterios($salvaCriterio);
                }
            } else {
                $salvaVistoria = array(
                    'DSOBSTAREFAGERCHK' => $parametros['DRG'],
                    'NRGERENCIAMENTOCHK' => $parametros['NR'],
                    "CDOPERULTATU"          => $operador,
                    "NRORGULTATU"           => $nrorg
                );

                $this->Fm00005000VistoriasFactory->salvarObservacao($salvaVistoria);

                $finalizaVistoria = array(
                    'NRGERENCIAMENTOCHK'    => $parametros['NR'],
                    "CDOPERULTATU"          => $operador,
                    "NRORGULTATU"           => $nrorg
                );

                $this->Fm00005000VistoriasFactory->finalizarVistoria($finalizaVistoria);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertVistoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg =  $this->Environment->getNrOrg();
            $operador = $this->Environment->getCdOperador();

            $nrgerenciamentochk = $this->newCode->getCodeByTableName("FCM_GERENCIAMENTOCHK", $nrorg, 1, 10, true);
            $DTHORAAGEND = $parametros['DATA']. " " . $parametros['HORARIO'];
            $DTHORAAGEND = new \DateTime($DTHORAAGEND);
            $params = array(
                "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                "NMGERENCIAMENTOCHK"    => $parametros['NMGERENCIAMENTOCHK'],
                "NRLOCALCLIENTE"        => $parametros['NRLOCALCLIENTE'],
                "NRCHECKLIST"           => $parametros['NRCHECKLIST'],
                "CDOPERADOR"            => $parametros['RESPONSAVEL'],
                "DTHORAAGEND"           => $DTHORAAGEND->format('d/m/Y H:i'),
                "OPERADOR"              => $operador,
                "IDOBRIGAOBSERVACAO"    => 'N',
                "NRSEQCONTRATO"         => (isset($parametros['NRSEQCONTRATO'])) ? $parametros['NRSEQCONTRATO'] : null
            );

            $this->Fm00005000VistoriasFactory->insertChk($params);

            foreach ($parametros['NRTAREFACHK'] as $atividade) {
                $nrtarefaagendchk = $this->newCode->getCodeByTableName("FCM_TAREFAGERCHK", $nrorg, 1, 10, true);
                $paramsTarefa = array(
                    "NRTAREFAGERCHK"        => $nrtarefaagendchk,
                    "NRGERENCIAMENTOCHK"    => $nrgerenciamentochk,
                    "NRTAREFACHK"           => $atividade,
                    "CDOPERADOR"            => $parametros['RESPONSAVEL'],
                    "DTHORATAREFAGERCHK"    => $DTHORAAGEND->format('d/m/Y H:i'),
                    "DSOBSTAREFAGERCHK"     => null,
                    "OPERADOR"              => $operador
                );

                $this->Fm00005000VistoriasFactory->insertChkTarefa($paramsTarefa);

                foreach ($parametros['NRCRITERIOTAREFACHK'] as $criterios) {
                    $paramsValida = array(
                        "NRTAREFACHK"      => $atividade,
                        "NRCRITERIOTAREFA" => $criterios['NRCRITERIOTAREFA']
                    );

                    $valida = $this->Fm00005000VistoriasFactory->validaCriterio($paramsValida);

                    if ($valida > 0) {
                        $nrcriteriostaragerchk = $this->newCode->getCodeByTableName("FCM_CRITERIOTARGERCHK", $nrorg, 1, 10, true);
                        $paramsCriterio = array(
                            "NRCRITERIOTARGERCHK" => $nrcriteriostaragerchk,
                            "NMCRITERIOTARGERCHK" => $criterios['DSCRITERIOTAREFA'],
                            "NRTAREFAGERCHK"      => $nrtarefaagendchk,
                            "NRCRITERIOTAREFACHK" => ($criterios['NRCRITERIOTAREFACHK'] === $criterios['DSCRITERIOTAREFA']) ? null : $criterios['NRCRITERIOTAREFACHK'],
                            "OPERADOR"            => $operador
                        );

                        $this->Fm00005000VistoriasFactory->insertChkCriterio($paramsCriterio);
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getResponsaveisVistoria($parametros)
    {
        if (!empty($parametros['NRLOCALCLIENTE'])) {
            $locais = is_array($parametros['NRLOCALCLIENTE']) ? $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']) : "'" . $parametros['NRLOCALCLIENTE'] . "'";
            return $this->Fm00005000VistoriasFactory->getResponsaveisVistoria($locais, array(
                "CDOPERADOR" => $this->Environment->getCdOperador(),
            ));
        }
    }
}
