<?php

namespace FM\Service;

use FM\Factory\LocaisAtendimentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;

class LocaisAtendimentoService
{

    protected $LocaisAtendimentoFactory;
    protected $FindService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, LocaisAtendimentoFactory $LocaisAtendimentoFactory, FindService $FindService, Environment $Environment, NewCode $newCode)
    {
        $this->connection                = $entityManager->getConnection();
        $this->LocaisAtendimentoFactory  = $LocaisAtendimentoFactory;
        $this->FindService               = $FindService;
        $this->Environment               = $Environment;
        $this->newCode                   = $newCode;
    }

    /* INTEGRAÇÃO COM SM */
    public function getContratosConfigurarLocais()
    {
        return $this->LocaisAtendimentoFactory->getContratosConfigurarLocais();
    }

    public function getLocaisSM($parametros)
    {
        $arrayFormatado = $this->FindService->formatarArray($parametros['NRSEQCONTRATO']);
        return $this->LocaisAtendimentoFactory->getLocaisSM($arrayFormatado);
    }

    public function salvarLocalSm($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = array();
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros as $arrayLocal) {
                $codNovoLocal = '';
                $localExistente = $this->LocaisAtendimentoFactory->verificarExistenciaLocais($arrayLocal['NRESTRUTURAM']);
                $contratoJaRelacionado = $this->LocaisAtendimentoFactory->verificarExistenciaContratoRelacionado(array(
                    "NRSEQCONTRATO"     => $arrayLocal['NRSEQCONTRATO'],
                    "NRESTRUTURAM"      => $arrayLocal['NRESTRUTURAM']));

                // Verifica se o local já foi inserido; caso o valor seja igual a zero, indica que ainda não foi inserido.
                if (count($localExistente) <= 0) {
                    $cdCliente = $this->LocaisAtendimentoFactory->getClienteContrato($arrayLocal['NRSEQCONTRATO']);

                    $params = array(
                        "NMLOCALCLIENTE"    => $arrayLocal['NMESTRUTURAM'],
                        "CDCLIENTE"         => $cdCliente,
                        "NRTIPOLOCALCLI"    => $arrayLocal['NRTIPOLOCALCLI'],
                        "NRESTRUTURAM"      => $arrayLocal['NRESTRUTURAM'],
                        "NRLOCALCLIENTEPAI" => null,
                        "NRCHECKLISTPADR"   => null,
                        "DSQRCODE"          => null,
                        "DSCODIGOLOCALCLI"  => $arrayLocal['DSCODIGOLOCALCLI'],
                        "DSCORIDLOCAL"      => $arrayLocal['DSCORIDLOCAL'],
                        "NMABREVLOCALCLI"   => $arrayLocal['NMABREVLOCALCLI']
                    );

                    $codNovoLocal = $this->inserirNovoLocal($params);
                } else {
                    array_push($retorno, $arrayLocal['NMESTRUTURAM']);
                }

                if ($contratoJaRelacionado == 0) {
                    $this->LocaisAtendimentoFactory->inserirRelacaoLocalContrato(array(
                        "NRRELACLOCALCONT"  => $this->newCode->getCodeByTableName('FCM_RELACLOCALCONT', $nrorg, 1, 10, true),
                        "NRLOCALCLIENTE"    => ($codNovoLocal === '') ? $localExistente[0]['NRLOCALCLIENTE'] : $codNovoLocal,
                        "NRRELACESTRUTCONT" => $arrayLocal['NRRELACESTRUTCONT'],
                        "NRSEQCONTRATO"     => $arrayLocal['NRSEQCONTRATO'],
                        "NRESTRUTURAM"      => $arrayLocal['NRESTRUTURAM'],
                        "NRORG"             => $nrorg,
                        "CDOPERINCLUSAO"    => $cdOperador,
                        "NRORGINCLUSAO"     => $nrorg,
                        "CDOPERULTATU"      => $cdOperador,
                        "NRORGULTATU"       => $nrorg
                    ));
                }
            }
            $this->connection->commit();
            return $retorno;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getLocaisContratosRelacionados($parametros)
    {
        return $this->LocaisAtendimentoFactory->getContratosRelacionadosLocais(array(
            "NRESTRUTURAM"  => $parametros['NRESTRUTURAM'],
        ));
    }

    /* FIM INTEGRAÇÃO COM SM */

    public function getClientes()
    {
        return $this->LocaisAtendimentoFactory->getClientes();
    }

    public function getLocais($param)
    {
        $filter = "";
        $params = explode(' ', isset($param['CDCLIENTE']) ? $param['CDCLIENTE'] : '');
        $filter = "AND LC.CDCLIENTE = '$params[0]'";
        return $this->LocaisAtendimentoFactory->getLocais(isset($param['CDCLIENTE']) ? $filter : "");
    }

    public function getTiposDeLocais($parametros)
    {

        $param = array(
            "IDTIPOLOCALCLI" => ($parametros['IDTIPOLOCALCLI'] === "0") ? null : $parametros['IDTIPOLOCALCLI']
        );

        return $this->LocaisAtendimentoFactory->getTiposDeLocais($param);
    }

    public function getChecklistPadrao()
    {
        return $this->LocaisAtendimentoFactory->getChecklistPadrao();
    }

    public function getSetorLocalAtendimento()
    {
        return $this->LocaisAtendimentoFactory->getSetorLocalAtendimento();
    }

    public function inserirNovoLocal($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $nrLocalCliente = $this->newCode->getCodeByTableName('FCM_LOCALCLI', $nrorg, 1, 10, true);
            $cdOperador = $this->Environment->getCdOperador();

            $p = array(
                "CDCLIENTE"         => $parametros['CDCLIENTE']
            );

            $cor = $this->LocaisAtendimentoFactory->getCorCliente($p);
            $params = array(
                "NRLOCALCLIENTE"    => $nrLocalCliente,
                "NMLOCALCLIENTE"    => $parametros['NMLOCALCLIENTE'],
                "CDCLIENTE"         => $parametros['CDCLIENTE'],
                "NRTIPOLOCALCLI"    => $parametros['NRTIPOLOCALCLI'],
                "NRLOCALCLIENTEPAI" => $parametros['NRLOCALCLIENTEPAI'],
                "NRCHECKLISTPADR"   => $parametros['NRCHECKLISTPADR'],
                "DSQRCODE"          => null,
                "NRORG"             => $nrorg,
                "CDOPERINCLUSAO"    => $cdOperador,
                "NRORGINCLUSAO"     => $nrorg,
                "DSCODIGOLOCALCLI"  => $parametros['DSCODIGOLOCALCLI'],
                "DSCORIDLOCAL"      => (!empty($cor)) ? $cor[0]['DSCORIDLOCAL'] : $parametros['DSCORIDLOCAL'],
                "NMABREVLOCALCLI"   => $parametros['NMABREVLOCALCLI'],
                "NRESTRUTURAM"      => $parametros['NRESTRUTURAM'],
            );

            $this->LocaisAtendimentoFactory->inserirNovoLocal($params);

            if ($parametros['NRCHECKLISTPADR'] !== null) {
                $nrRelacLocalChk = $this->newCode->getCodeByTableName('FCM_RELACLOCALCHK', $nrorg, 1, 10, true);

                $paramsChk = array(
                    "NRRELACLOCALCHK" => $nrRelacLocalChk,
                    "NRLOCALCLIENTE"  => $nrLocalCliente,
                    "NRCHECKLIST"     => $parametros['NRCHECKLISTPADR']
                );

                $this->LocaisAtendimentoFactory->inserirRelacLocalChk($paramsChk);
            }

            $this->connection->commit();
            return $nrLocalCliente;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getClientesQueAgrupamLocais()
    {
        return $this->LocaisAtendimentoFactory->getClientesQueAgrupamLocais();
    }

    public function getLocaisAgrupados($parametro)
    {
        // descomentar essa linha caso seja necessário aplicar o fetchOnDemand como true no frontend
        $param = array(
            "CDCLIENTE" => $parametro['CDCLIENTE'],
            // "NRLOCALCLIENTEPAI" => ($parametro['NRLOCALCLIENTEPAI'] === 'NULL') ? null : $parametro['NRLOCALCLIENTEPAI']
        );

        return $this->LocaisAtendimentoFactory->getLocaisAgrupados($param);
    }

    public function alterarCorCliente($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();

            $param = array(
                "DSCORIDLOCAL" => $parametros['DSCORIDLOCAL'],
                "CDCLIENTE"    => $parametros['CDCLIENTE'],
                "NRORGULTATU"  => $nrorg,
                "CDOPERULTATU" => $cdOperador,

            );

            $this->LocaisAtendimentoFactory->alterarCorCliente($param);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function atualizarLocal($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();

            $p = array(
                "CDCLIENTE"         => $parametros['CDCLIENTE']
            );

            $cor = $this->LocaisAtendimentoFactory->getCorCliente($p);

            $params = array(
                "NRLOCALCLIENTE"    => $parametros['NRLOCALCLIENTE'],
                "NMLOCALCLIENTE"    => $parametros['NMLOCALCLIENTE'],
                "CDCLIENTE"         => $parametros['CDCLIENTE'],
                "NRTIPOLOCALCLI"    => $parametros['NRTIPOLOCALCLI'],
                "NRLOCALCLIENTEPAI" => $parametros['NRLOCALCLIENTEPAI'],
                "NRCHECKLISTPADR"   => $parametros['NRCHECKLISTPADR'],
                "NMABREVLOCALCLI"   => $parametros['NMABREVLOCALCLI'],
                "DSCODIGOLOCALCLI"  => $parametros['DSCODIGOLOCALCLI'],
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdOperador,
            );

            $this->LocaisAtendimentoFactory->atualizarLocal($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarLocais($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $cont = 0;
            $contLocaisFilhos = 0;
            $filtrados = '';
            $nrorg      = $this->Environment->getNrorg();
            $cdOperador = $this->Environment->getCdOperador();


            while ($cont <= 2) {
                $indice = (3 - $cont);

                switch ($indice) {
                    case 1:
                        $filtrados = $parametros['TIPO1'];
                        break;
                    case 2:
                        $filtrados = $parametros['TIPO2'];
                        break;
                    case 3:
                        $filtrados = $parametros['TIPO3'];
                        break;
                }

                foreach ($filtrados as $filtrado) {
                    $filhos = $this->LocaisAtendimentoFactory->verificarExistenciaFilhos($filtrado);

                    if ($filhos == 0) {
                        $params = array(
                            "NRLOCALCLIENTE" => $filtrado,
                            "NRORGULTATU"    => $nrorg,
                            "CDOPERULTATU"   => $cdOperador,
                        );

                        $this->LocaisAtendimentoFactory->inativarLocais($params);
                    } else {
                        $contLocaisFilhos++;
                    }
                }
                $cont++;
            }

            $this->connection->commit();
            return $mensagem = ($contLocaisFilhos == 0) ? 'Ok' : 'err';
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    // --- Funções da TAB DE TIPO DE LOCAL
    public function getTipoDeLocal()
    {
        return $this->LocaisAtendimentoFactory->getTipoDeLocal();
    }

    public function excluirTipoLocal($params)
    {

        $this->connection->beginTransaction();

        try {
            foreach ($params['NRS'] as $chave => $linha) {
                $dataParam = array("NRTIPOLOCALCLI" => $linha);
                $this->LocaisAtendimentoFactory->excluirTipoLocal($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function atualizarTipoLocal($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["NRTIPOLOCALCLI"] != 0) {
                    $parametros = array(
                        "NRTIPOLOCALCLI" => $row["NRTIPOLOCALCLI"],
                        "NMTIPOLOCALCLI" => $row['NMTIPOLOCALCLI'],
                        "IDTIPOLOCALCLI" => $row['IDTIPOLOCALCLI'],
                        "NRORGULTATU"    => $nrorg,
                        "CDOPERULTATU"   => $operador
                    );

                    $this->LocaisAtendimentoFactory->atualizarTipoLocal($parametros);
                } else {
                    $nrtipolocalcli = $this->newCode->getCodeByTableName('FCM_TIPOLOCALCLI', $nrorg, 1, 10, true);
                    $parametros = array(
                        "NRTIPOLOCALCLI" => $nrtipolocalcli,
                        "NMTIPOLOCALCLI" => $row['NMTIPOLOCALCLI'],
                        "IDTIPOLOCALCLI" => $row['IDTIPOLOCALCLI'],
                        "NRORG"          => $nrorg,
                        "NRORGINCLUSAO"  => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "IDATIVO"        => "S"
                    );

                    $this->LocaisAtendimentoFactory->inserirTipoLocal($parametros);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // ----------- FUNÇÕES GEOLOCALIZAÇÃO ------------
    public function salvarGeolocalizacao($param)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            $nr = array();
            $filtrar = "";
            $nrLocal = $param['NRLOCAL'];

            // verifica se é AGCOMPLETO(1) e se possui filhos. Caso verdadeiro, busca o NR dos AGSIMPLES(2)
            if ($param['LEVELTREE'] == 1 && $param['CHILDREN'] > 0) {
                $nrAgrupamentoSimples = $this->LocaisAtendimentoFactory->buscarNrAgrupamentoSimples($param['NRLOCAL']);
                foreach ($nrAgrupamentoSimples as $agrupamentoSimples) {
                    array_push($nr, $agrupamentoSimples['NRLOCALCLIENTE']);
                }
                array_push($nr, $param['NRLOCAL']);
                $nrPai = implode(', ', $nr);

                // Verifica se é AGCOMPLETO e se não possui filhos.
            } elseif ($param['LEVELTREE'] == 1 && $param['CHILDREN'] == 0) {
                $nrPai = $param['NRLOCAL'];

                // Verifica se é AGSIMPLES e se possui filhos.
            } elseif ($param['LEVELTREE'] == 2 && $param['CHILDREN'] > 0) {
                $nrPai = $param['NRLOCAL'];
            }

            // Se for AGCOMPLETO e não tiver filhos ou se for AGSIMPLES e não tiver filhos, setará LatLng nos respectivos AG.
            if ($param['LEVELTREE'] == 1 && $param['CHILDREN'] == 0 || $param['LEVELTREE'] == 2 && $param['CHILDREN'] == 0) {
                $filtrar .= "WHERE NRLOCALCLIENTE = $nrLocal";
               
                // Se não for LCATEND(3), vai setar lat e lng nos filhos do agrupamento escolhido (AGCOMPLETO ou AGSIMPLES) e nele próprio.
            } elseif ($param['LEVELTREE'] != 3) {
                $filtrar .= "WHERE NRLOCALCLIENTEPAI IN ($nrPai) OR NRLOCALCLIENTE = $nrLocal";

                // Se foi escolhido LCATEND(3), vai setar let e lng apenas no local escolhido.
            } elseif ($param['LEVELTREE'] == 3) {
                $filtrar .= "WHERE NRLOCALCLIENTE = $nrLocal";
            }

            $parametros = array(
                "NRGEOLOCALIZACAO"  => $param['CIRCLE'],
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $operador
            );
            $this->LocaisAtendimentoFactory->salvarGeolocalizacao($parametros, $filtrar);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getGeolocalizacao($param)
    {
        return $this->LocaisAtendimentoFactory->getGeolocalizacao($param['NRLOCALCLIENTE']);
    }
}
