import { Utilities } from '@/util/Utilities';
import {
    Button, Checkbox, FileInput, Form, LoadingService, Modal, ModalService, SelectMultiple, Text, Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata, dayjs,
} from '@zeedhi/core';
import { IterableCarousel } from '@zeedhi/teknisa-components-common';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';
import FileUpload from '../FileUpload';

export class RelatorioAuditoriasController {
    private visibilidadePaginaInicial: boolean = true;

    private visibilidadePaginaCateg: boolean = false;

    private modalDadosAuditoria!: Modal;

    private modalFiltrarAuditoria!: Modal;

    private modalAnexarImagem!: Modal;

    private nomeAuditoria: String = '';

    public visibilidadeCarroselImagens: boolean = false;

    private modalExibirImagem!: Modal;

    public stylizeStatus({ component }: IEventParam<Text>) {
        if (component.text === 'Pendente') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
        }
        if (component.text === 'Concluído') {
            component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
        }
        if (component.text === 'Em andamento') {
            component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
    }

    private currentCard: any = '';

    public clickCard({ component }: IEventParam<TekIterableComponentRender>) {
        this.visibilidadePaginaInicial = false;
        this.visibilidadePaginaCateg = !this.visibilidadePaginaInicial;
        /* eslint-disable */
        this.currentCard = this.getDadosCardClicado(component.name.split('_')[1]);
        /* eslint-enable */
        this.nomeAuditoria = this.currentCard.NMAUDITORIA;
        this.onMountedCategorias();
        this.onMountedValidacoes();
    }

    public async criarModais() {
        this.modalDadosAuditoria = await ModalService.createFromJson('visualizar_infos_relatorio_auditorias', true);
        // this.modalDadosAuditoria = await ModalService.createFromJson('fm_00028_000_relatorios/modal/fm_00040_000_dados_auditoria', true);
        // this.modalFiltrarAuditoria = await ModalService.createFromJson('fm_00028_000_relatorios/modal/fm_00040_000_filtrar_auditorias', true);
        // this.modalAnexarImagem = await ModalService.createFromJson('fm_00028_000_relatorios/modal/fm_00040_000_anexar_imagens', true);
    }

    public destruirModais() {
        ModalService.delete(this.modalDadosAuditoria);
        ModalService.delete(this.modalFiltrarAuditoria);
        ModalService.delete(this.modalAnexarImagem);
    }

    public openBasicModal_WWFS() {
        this.modalDadosAuditoria.show();
      }

    public abrirModalInfos() {
        this.modalDadosAuditoria.show();
    }

    public fecharModalInfos() {
        this.modalDadosAuditoria.hide();
    }

    public abrirFiltros() {
        this.modalFiltrarAuditoria.show();
    }

    public fecharFiltros() {
        this.modalFiltrarAuditoria.hide();
    }

    private nrcategoria: any = '';

    public abrirAnexoImagem({ component }: any) {
        this.modalAnexarImagem.show();
        /* eslint-disable */
        this.nrcategoria = component.name.split('_')[1];
        /* eslint-enable */
    }

    public fecharAnexoImagem() {
        this.modalAnexarImagem.hide();
    }

    private nr: any = '';

    public async abrirModalExibirImagens({ component }: any) {
        this.modalExibirImagem = await ModalService.createFromJson('fm_00028_000_relatorios/modal/fm_00040_000_visualizar_imagens', true);
        this.modalExibirImagem.show();
        /* eslint-disable */
        this.nr = component.name.split('_')[1];
        /* eslint-enable */
    }

    public fecharModalExibirImagens() {
        this.modalExibirImagem.hide();
        ModalService.delete(this.modalExibirImagem);
    }

    public getDadosCardClicado(nrAuditoria: any) {
        const { data } = Metadata.getInstance<TekIterableComponentRender>('iterable-minhas-auditoria').datasource;
        const dados: any = data?.find((row: any) => row.NRAUDITORIA === Number(nrAuditoria));
        return dados;
    }

    public exibirInfos({ component }: IEventParam<Button>) {
        const dados = this.getDadosCardClicado(component.name.split('_')[1]);

        this.abrirModalInfos();
        LoadingService.show();
        setTimeout(() => {
            Metadata.getInstance<Text>('titulo-dados-auditoria').text = dados.NMAUDITORIA;
            Metadata.getInstance<Text>('info-local').text = dados.NMLOCALCLIENTE;
            Metadata.getInstance<Text>('info-cliente').text = dados.NMFANTCLIE;
            Metadata.getInstance<Text>('info-contrato').text = (dados.NRCONTRATOFCT === null) ? 'Não há' : dados.NRCONTRATOFCT;
            Metadata.getInstance<Text>('info-data').text = dados.DTAGEND;
            Metadata.getInstance<Text>('info-funcionario').text = dados.NMOPERADOR;
            Metadata.getInstance<Text>('info-situacao').text = dados.IDSITUACAOAUDIT;
        }, 500);
        LoadingService.hide();
    }

    public limparFiltros() {
        Metadata.getInstance<Form>('form-filtros').clearForm();
        Metadata.getInstance<TekIterableComponentRender>('iterable-minhas-auditoria').datasource.get();
    }

    public filtrarAuditorias() {
        LoadingService.show();
        Metadata.getInstance<TekIterableComponentRender>('iterable-minhas-auditoria').datasource.setFilter({
            NRMODELOAUDIT: Metadata.getInstance<SelectMultiple>('filtro-modelo').value,
            CDCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-cliente').value,
            NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('filtro-contratos').value,
            NMLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            DTAGEND: Metadata.getInstance<SelectMultiple>('data').value,
        });
        this.fecharFiltros();
        LoadingService.hide();
    }

    /* Funções da tela de execução */

    public validationsTextArea({ component }: IEventParam<Textarea>) {
        if (this.currentCard.IDOBRIGAOBSERVACAO === 'S') {
            component.addValidation('required', {});
        } else {
            component.removeValidation('required');
        }
    }

    public botaoFinalizar: boolean = true;

    public botaoIniciar: boolean = true;

    private botaoAnexar: boolean = true;

    private textArea: boolean = true;

    private checkbox: boolean = true;

    public onMountedValidacoes() {
        if (this.currentCard.IDSITUACAOAUDIT === 'Pendente') {
            this.botaoFinalizar = true;
            this.botaoIniciar = false;
            this.botaoAnexar = false;
            this.textArea = false;
            this.checkbox = false;
        } else if (this.currentCard.IDSITUACAOAUDIT === 'Em andamento') {
            this.botaoFinalizar = false;
            this.botaoIniciar = true;
            this.botaoAnexar = false;
            this.textArea = false;
            this.checkbox = false;
        } else {
            this.botaoFinalizar = true;
            this.botaoIniciar = true;
            this.botaoAnexar = true;
            this.textArea = true;
            this.checkbox = true;
        }
    }

    public retornarPagina() {
        this.visibilidadePaginaInicial = true;
        this.visibilidadePaginaCateg = !this.visibilidadePaginaInicial;
    }

    public onMountedCategorias() {
        Metadata.getInstance<TekIterableComponentRender>('iterable-categorias').datasource.setFilter({
            NRAUDITORIA: this.currentCard.NRAUDITORIA,
        });
    }

    public getTopicos({ component }: IEventParam<TekIterableComponentRender>) {
        component.datasource.setFilter({
            NRCATEGORIAAUDIT: component.name.split('_')[1],
            NRAUDITORIA: this.currentCard.NRAUDITORIA,
        });
    }

    public getCriterios({ component }: IEventParam<TekIterableComponentRender>) {
        component.datasource.setFilter({
            NRTOPICOAUDIT: component.name.split('_')[1],
            NRAUDITORIA: this.currentCard.NRAUDITORIA,
        });
    }

    /* Nesta função também contém a tratativa para alterar o status do Item (IDREALIZADO) para 'S'
    tanto no momento da iniciação automática quanto em uma tarefa já em andamento */
    public changeCheckbox({ component }: any) {
        const iterableCheckbox = Metadata.getInstance<TekIterableComponentRender>(`iterable-criterios_${component.name.split('_')[2]}`);
        const iterableTopicos = Metadata.getInstance<TekIterableComponentRender>(`iterable-topicos_${component.name.split('_')[3]}`);

        const chkboxS: IDictionary = [];
        const chkboxN: IDictionary = [];

        iterableTopicos.datasource.data?.forEach((topic: IDictionary) => {
            /* eslint-disable */
            if (topic.NRTOPICOAUDIT === parseInt(component.name.split('_')[2])) {
                iterableCheckbox.datasource.data?.forEach((itens: IDictionary) => {
                    const checkbox = Metadata.getInstance<Checkbox>(`checkbox_${itens.NRRELACRESULTCRIT}_${itens.NRTOPICOAUDIT}_${itens.NRCATEGORIAAUDIT}`);
                    if (this.currentCard.IDTIPOMARCACAO === 'U' && checkbox.name !== component.name && checkbox.value === 'S') {
                        checkbox.value = 'N';
                    }

                    if (checkbox.value === 'S') {
                        chkboxS.push(itens.NRRELACRESULTCRIT);
                    } else {
                        chkboxN.push(itens.NRRELACRESULTCRIT);
                    }
                });
            }
            /* eslint-enable */
        });

        if (this.currentCard.IDSITUACAOAUDIT === 'Pendente') {
            this.iniciarOuFinalizarAuditoria('2');
            this.botaoIniciar = true;
            this.botaoFinalizar = false;
            this.alterarStatusCriterio(chkboxS, chkboxN);
        } else if (this.currentCard.IDSITUACAOAUDIT === 'Em andamento') {
            this.alterarStatusCriterio(chkboxS, chkboxN);
        }
    }

    public iniciarOuFinalizarBotao({ component }: IEventParam<Button>) {
        const id = (component.name === 'botao-iniciar') ? '2' : '3';
        this.iniciarOuFinalizarAuditoria(id);
    }

    public iniciarOuFinalizarAuditoria(id: any) {
        const iterableAuditorias = Metadata.getInstance<TekIterableComponentRender>('iterable-minhas-auditoria');

        LoadingService.show();
        Http.post('/inicarOuFinalizarAuditoria', {
            IDSITUACAOAUDIT: id,
            NRAUDITORIA: this.currentCard.NRAUDITORIA,
        }).then(() => {
            iterableAuditorias.datasource.get();
            Utilities.showSuccessMessage();
            if (id === '3') {
                this.botaoFinalizar = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public alterarStatusCriterio(chkboxS: any, chkboxN: any) {
        Http.post('/alterarStatusCriterio', {
            criteriosN: chkboxN,
            criteriosS: chkboxS,
        }).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public exibirImagemAnexada({ component }: any) {
        const { datasource } = Metadata.getInstance<IterableCarousel>('carousel-imagens').iterable;
        const imagens: any = [];

        if (component.value.length) {
            this.visibilidadeCarroselImagens = true;
            component.value.forEach((elemento: any, posicao: any) => {
                imagens.push({
                    id: posicao,
                    src: window.URL.createObjectURL(elemento),
                });
            });
        } else {
            this.visibilidadeCarroselImagens = false;
        }

        datasource.updateData(imagens);
    }

    public async anexarImagens() {
        const imagensConvertidas = await FileUpload.converterImagens('file-input-anexar-imagem');
        const fileInput = Metadata.getInstance<FileInput>('file-input-anexar-imagem');
        if (fileInput.value) {
            Http.post('/anexarImagemAuditoria', {
                IMAGENS: imagensConvertidas,
                NRAUDITORIA: this.currentCard.NRAUDITORIA,
                NRCATEGORIAAUDIT: this.nrcategoria,
            }).then(() => {
                Utilities.showSuccessMessage();
                this.fecharAnexoImagem();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Nenhuma imagem foi anexada', 'error');
        }
    }

    public carregarFotosVistoria({ component }: any) {
        const btnApagar = Metadata.getInstance<Button>('botao-apagar-imagens');
        component.iterable.datasource.data = [];
        LoadingService.show();
        Http.post('/getImagensAuditoria', {
            NRAUDITORIA: this.nr,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
                btnApagar.isVisible = false;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTORESULTAUDIT: dados.NRFOTORESULTAUDIT,
                        DSFOTOAUDITORIA: dados.DSFOTOAUDITORIA,
                    });
                    btnApagar.isVisible = true;
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async excluirFoto() {
        const carrosel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-auditorias');
        const img = carrosel.iterable.datasource.data;
        const [deleteData] = img.filter((value: any, index: any) => index === carrosel.currentSlide);

        LoadingService.show();
        Http.post('/excluirImagemAuditoria', {
            IMG: deleteData,
        }).then(() => {
            const data = img.filter((value: any, index: any) => index !== carrosel.currentSlide);
            carrosel.iterable.datasource.updateData(data);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public tipoOrdenacao = '';

    public ordenarCards({ component }: IEventParam<Button>) {
        const iterable = Metadata.getInstance<TekIterableComponentRender>('iterable-minhas-auditoria');

        iterable.datasource.clearOrder();
        if (component.name === 'botao-ordenar-auditoria') {
            iterable.datasource.addOrder('NMAUDITORIA', 'ASC');
        } else if (component.name === 'botao-ordenar-local') {
            iterable.datasource.addOrder('NMLOCALCLIENTE', 'ASC');
        } else if (component.name === 'botao-ordenar-data') {
            iterable.datasource.addOrder('DTHORAAUDIT', 'DESC');
        } else {
            iterable.datasource.clearOrder();
        }

        const { order } = iterable.datasource;
        if (order) {
            const campo = (order.length === 0) ? '' : order[0].split('.')[0];
            this.exibirTipoOrdenacao(campo);
        }
        iterable.datasource.setPage(1);
        this.inserirOrdenacao({ component: iterable });

        iterable.datasource.setPage(1);
    }

    public exibirTipoOrdenacao(name: any) {
        const visibilidadeText = Metadata.getInstance<Text>('titulo-ordenacao');

        switch (name) {
            case 'NMAUDITORIA':
                this.tipoOrdenacao = 'Ordenado por nome da Auditoria';
                visibilidadeText.isVisible = true;
                break;
            case 'NMLOCALCLIENTE':
                this.tipoOrdenacao = 'Ordenado por Local';
                visibilidadeText.isVisible = true;
                break;
            case 'DTHORAAUDIT':
                this.tipoOrdenacao = 'Ordenado por Data';
                visibilidadeText.isVisible = true;
                break;
            default:
                this.tipoOrdenacao = '';
                visibilidadeText.isVisible = false;
                break;
        }
    }

    public inserirOrdenacao({ component }: IEventParam<TekIterableComponentRender>) {
        const { order } = component.datasource;
        if (order) {
            Http.post('/insertOrdenacao', {
                NMTELA: 'Auditoria de Tarefas',
                NMCAMPO: (order.length === 0) ? '' : order[0].split('.')[0],
                IDSENTIDO: (order.length === 0) ? '' : order[0].split('.')[1],
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public onMountedOrdenacao({ component }: IEventParam<TekIterableComponentRender>) {
        Http.post('/getOrdenacao', {
            NMTELA: 'Auditoria de Tarefas',
        }).then((response: IDictionary) => {
            if (response.data.length !== 0 && typeof response.data.length !== 'undefined') {
                component.datasource.clearOrder();
                component.datasource.addOrder(response.data[0].NMCAMPO, response.data[0].IDSENTIDO);
                this.exibirTipoOrdenacao(response.data[0].NMCAMPO);
                Utilities.showSuccessMessage();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public sortFunction(value1: any, value2: any, columnName: string): boolean {
        if (columnName === 'DTHORAAUDIT') {
          return (dayjs(value1, 'DD/MM/YYYY HH24:MI').isAfter(dayjs(value2, 'DD/MM/YYYY HH24:MI')));
        }
        return (value1 > value2);
    }

    public carregarCards({ component }: any) {
        setTimeout(() => {
            this.onMountedOrdenacao({ component });
        }, 500);
    }
}
